from typing import List, Tuple


class Injector:

    @staticmethod
    def inject(tagged_words: List[Tuple[str, str]], values: List[str], label: str) -> List[Tuple[str, str]]:
        """
        Tagged word contains list of the pairs.
        The method switches the pair like <LABEL, _> to the pair like <VALUE, LABEL> with the given values and label.

        Parameters
            tagged_words: List[Tuple[str, str]]
                list of words with labels
            values: List[str]
                list of values to insert instead of label
            label: str
                replaced tag

        Return
            List[Tuple[str, str]]
                tagged words (with replaced date tags)
        """
        counter = 0
        for i, x in enumerate(tagged_words):
            if x[0] == label:
                tagged_words[i] = (values[counter], label)
                counter += 1
        return tagged_words



"""
Wrapper over nltk.pos_tag
Allows to use custom part of speech tags
"""
import nltk
from typing import List, Tuple

from .injector import Injector
from .rule import Rule


def pos_tag(text: str, rules: List[Rule], lang: str = 'rus') -> List[Tuple[str, str]]:
    """
    Parameters
        text: str
            input text
        rules: List[nltk_custom.rule.Rule]
            rules for custom tags
        lang: str
            language of the input text

    Return
        List[Tuple[str, str]]
            list of tagged words.
    """

    # forward preprocess
    for i, rule in enumerate(rules):
        text, matches = rule.extractor.extract(text)
        rules[i] = rule._replace(matches=matches)

    # split to tags
    tagged_text = _tag_preprocess(text, lang=lang)

    # backward preprocess
    for rule in rules:
        tagged_text = Injector.inject(tagged_text, rule.matches, rule.extractor.keyword.strip())

    return tagged_text


def _tag_preprocess(text: str, lang: str):
    """
    Parameters
        text: str
            input text
        lang: str
            language of the input text

    Return
        List[Tuple[str, str]]
            tagged words of the input text
    """
    sentences = nltk.sent_tokenize(text)
    sentences = [nltk.word_tokenize(sent) for sent in sentences]
    sentences = [nltk.pos_tag(sent, lang=lang) for sent in sentences]
    return [y for x in sentences for y in x]



import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="cnpt_leos",
    version="0.1.1",
    author = "Dmitry Obukhov",
    author_email = "bstodin@gmail.com",
    description = ("Custom NLTK part of speech tagger"),
    long_description=read('README.md'),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)



"""
Abstract class
Describes the entity that extracts from the text the components that satisfy the rule.
"""


from typing import List, Tuple


class Extractor:

    def __init__(self, keyword: str):
        """
        Parameters
            keyword: str
                keyword for an extracted entities
        """
        pass

    def extract(self, text: str) -> Tuple[str, List[str]]:
        """
        Parameters
            text: str
                input text

        Return
            Tuple[str, List[str]]
                processed text and detected matches
        """
        raise NotImplementedError



import re

from typing import List, Tuple

from cnpt.extractor import Extractor
from cnpt.pos_tag import pos_tag
from cnpt.rule import Rule


# 1. Define an extractor

class RegExpExtractor(Extractor):

    def __init__(self, keyword: str, pattern: str):
        super().__init__(keyword)
        self.keyword = keyword
        self.pattern = pattern

    def extract(self, text: str) -> Tuple[str, List[str]]:
        matches = re.findall(self.pattern, text)
        matches = [x[0] for x in matches]
        for m in matches:
            text = text.replace(m, self.keyword, 1)
        return text, matches

greeting_extractor = RegExpExtractor(keyword='GREETING', pattern=r"((Hello|Hi))")


# 2. Define rules

rule = Rule(extractor=greeting_extractor, matches=None)
rules = [rule]


# 3. Use pos_tag to make a tagged words from text

text = 'Hello world!'
tagged_words = pos_tag(text=text, rules=rules)


# 4. See result

print(text)
print(tagged_words)
